@@ -22,8 +22,8 @@
 #include <mgui/_pc_.h>
 
 #include "script.h"
-#include "execute.h"
 #include "burn.h"
+#include "ffmpeg.h"
 
 #include <mgui/init.h>
 #include <mgui/render/menu.h>
@@ -31,6 +31,8 @@
 #include <mgui/sdk/widget.h>
 #include <mgui/sdk/textview.h>
 #include <mgui/project/thumbnail.h> // Project::CalcAspectSize()
+#include <mgui/project/video.h>
+#include <mgui/project/handler.h>
 #include <mgui/gettext.h>
 
 #include <mbase/project/table.h>
@@ -41,6 +43,7 @@
 #include <mlib/filesystem.h>
 #include <mlib/string.h>
 #include <mlib/sdk/logger.h>
+#include <mlib/regex.h>
 
 #include <boost/lexical_cast.hpp>
 
@@ -66,19 +69,12 @@ TSBegEnd BeginEnd(RefPtr<Gtk::TreeStore> store)
     return std::make_pair(children.begin(), children.end());
 }
 
-bool CheckAuthorMode::Flag = false;
-
-CheckAuthorMode::CheckAuthorMode(bool turn_on): origVal(Flag) 
-{ Flag = turn_on; }
-CheckAuthorMode::~CheckAuthorMode()
-{ Flag = origVal; }
-
 // из-за того, что все медиа в одном браузере (включая картинки="не видео"),
 // нельзя использовать естественную нумерацию для авторинга 
-static int& GetAuthorNumber(VideoMD& obj)
+static int& GetAuthorNumber(VideoItem vi)
 {
     //return GetBrowserPath(&obj)[0]+1;
-    return obj.GetData<int>(AUTHOR_TAG);
+    return vi->GetData<int>(AUTHOR_TAG);
 }
 
 int ForeachVideo(VideoFnr fnr)
@@ -111,17 +107,6 @@ void ForeachMenu(MenuFnr fnr)
     }
 }
 
-bool IndexForAuthoring(VideoItem vi, int idx)
-{
-    GetAuthorNumber(*vi) = idx+1; // индекс от 1
-    return true;
-}
-
-static int IndexVideosForAuthoring()
-{
-    return ForeachVideo(bl::bind(&IndexForAuthoring, bl::_1, bl::_2));
-}
-
 static std::string MakeFPTarget(MediaItem mi)
 {
     // Doc: "DVD-Video/2. The Unofficial DVD Specifications Guide/1. Guide - DVD2.2.5.pdf",
@@ -137,7 +122,7 @@ static std::string MakeFPTarget(MediaItem mi)
     {
         VideoItem vi = IsVideo(mi);
         ASSERT( vi );
-        str = (str::stream() << "title " << GetAuthorNumber(*vi)).str();
+        str = (str::stream() << "title " << GetAuthorNumber(vi)).str();
     }
     return str;
 }
@@ -166,14 +151,19 @@ void TargetCommandVis::Visit(MenuMD& obj)
     res = boost::format("g1 = %1%; %2% menu entry root;") % GetMenuANum(&obj) % (vtsDomain ? "call" : "jump") % bf::stop; 
 }
 
+static std::string JumpTitleCmd(int num)
+{
+    return boost::format("jump title %1%;") % num % bf::stop; 
+}
+
 void TargetCommandVis::Visit(VideoMD& obj)
 {
-    res = (str::stream() << "jump title " << GetAuthorNumber(obj) << ";").str(); 
+    res = JumpTitleCmd(GetAuthorNumber(&obj)); 
 }
 
 void TargetCommandVis::Visit(VideoChapterMD& obj) 
 {
-    int v_num = GetAuthorNumber(*obj.owner);
+    int v_num = GetAuthorNumber(obj.owner);
     // :TODO: title 1 всегда равно title 1 chapter 1; при этом dvdauthor не воспринимает 
     // главы "вблизи нуля" (<0.27секунд; задание - посмотреть точно), поэтому нужна предварительная
     // перенумерация, как с видео
@@ -187,12 +177,6 @@ static std::string MakeButtonJump(MediaItem mi, bool vts_domain)
     return vis.res;
 }
 
-static bool _GetFirstVideo(VideoItem& res, VideoItem vi, int)
-{
-    res = vi;
-    return false;
-}
-
 std::string MenuAuthorDir(Menu mn, int idx, bool cnv_from_utf8)
 {
     // нужно безопасное имя для директории
@@ -212,14 +196,31 @@ std::string MenuAuthorDir(Menu mn, int idx, bool cnv_from_utf8)
     return cnv_from_utf8 ? ConvertPathFromUtf8(fname) : fname ;
 }
 
+int TitlesCount()
+{
+    return boost::distance(AllVideos());
+}
+
+static std::string PlayAllCmd()
+{
+    if( TitlesCount() == 0 )
+        Author::Error("There is no video for \"Play All\" function.");
+    return boost::format("g2 = 1; %1%") % JumpTitleCmd(1) % bf::stop;
+}
+
 bool HasButtonLink(Comp::MediaObj& m_obj, std::string& targ_str)
 {
-    bool res = false;
-    if( MediaItem btn_target = m_obj.MediaItem() )
+    bool res = true;
+
+    if( m_obj.PlayAll() )
+        targ_str = PlayAllCmd();
+    else if( MediaItem btn_target = m_obj.MediaItem() )
     {
         targ_str = MakeButtonJump(btn_target, false);
         res = !targ_str.empty();
     }
+    else
+        res = false;
     return res;
 }
 
@@ -228,6 +229,90 @@ int MenusCnt()
     return Size(GetAStores().mnStore);
 }
 
+static bool IsEndVideo(int cur_num)
+{
+    return cur_num == TitlesCount();
+}
+
+static std::string JumpNextTitle(VideoItem vi)
+{
+    int cur_num = GetAuthorNumber(vi);
+    ASSERT( (cur_num > 0) && (cur_num <= TitlesCount()) );
+
+    int next_num = IsEndVideo(cur_num) ? 1 : cur_num + 1;
+    return JumpTitleCmd(next_num);
+}
+
+static std::string AutoPostCmd(const std::string& jnt_cmd)
+{
+    return MenusCnt() ? std::string("call menu entry root;") : jnt_cmd ;
+}
+
+static void AddChildWithText(xmlpp::Element* node, const char* node_name,
+                             const std::string& body)
+{
+    node->add_child(node_name)->add_child_text(body);
+}
+
+static void AddPostCmd(xmlpp::Element* pgc_node, MediaItem mi)
+{
+    VideoItem vi = IsVideo(mi);
+    Menu mn      = IsMenu(mi);
+
+    ASSERT_RTL( vi || mn );
+    // VTS domain
+    bool is_video = vi;
+    const PostAction& pa = is_video ? vi->PAction() : mn->MtnData().pAct ;
+
+    std::string jnt_cmd;
+    // авто-команда
+    std::string auto_cmd;
+    if( is_video )
+    {
+        jnt_cmd = JumpNextTitle(vi);
+        // чаще всего будет эта команда
+        auto_cmd = AutoPostCmd(jnt_cmd);
+    }
+    else
+        auto_cmd = "jump cell 1;"; // Loop
+
+    std::string post_cmd;
+    switch( pa.paTyp )
+    {
+    case patAUTO:
+        post_cmd = auto_cmd;
+        break;
+    case patNEXT_TITLE:
+        ASSERT( is_video );
+        post_cmd = jnt_cmd;
+        break;
+    case patEXP_LINK:
+        if( pa.paLink )
+            post_cmd = MakeButtonJump(pa.paLink, is_video);
+        break;
+    case patPLAY_ALL:
+        post_cmd = PlayAllCmd();
+        break;
+    default:
+        //ASSERT_RTL(0);
+        break; // не вылетаем на плохих проектах
+    }
+    if( post_cmd.empty() ) // пока "Stop" нет, и непонятно, как лучше его сделать
+        post_cmd = auto_cmd;
+
+    if( is_video )
+    {
+        // реализация Play All
+        std::string prefix("g2 = 0;");
+        if( !IsEndVideo(GetAuthorNumber(vi)) )
+            prefix = boost::format("if(g2 == 1) {%1%}") % jnt_cmd % bf::stop;
+
+        post_cmd = prefix + " " + post_cmd;
+    }
+
+    AddChildWithText(pgc_node, "post", post_cmd);
+}
+
 static bool ScriptMenu(xmlpp::Element* menus_node, Menu root_menu, Menu mn, int i)
 {
     xmlpp::Element* pgc_node = menus_node->add_child("pgc");
@@ -239,13 +324,16 @@ static bool ScriptMenu(xmlpp::Element* menus_node, Menu root_menu, Menu mn, int
         int num = GetMenuANum(mn);
         int next_num = (i+1 < MenusCnt()) ? num+1 : 1 ;
         std::string loop_menus = boost::format("if(g1 != %1%) {jump menu %2%;}") % num % next_num % bf::stop;
-        pgc_node->add_child("pre")->add_child_text(loop_menus);
+        AddChildWithText(pgc_node, "pre", loop_menus);
     }
 
     xmlpp::Element* vob_node = pgc_node->add_child("vob");
     // название меню
     std::string m_dir = MenuAuthorDir(mn, i);
     vob_node->set_attribute("file", AppendPath(m_dir, "MenuSub.mpg"));
+    if( IsMotion(mn) )
+        AddPostCmd(pgc_node, mn);
+
     // действия кнопок
     MenuRegion::ArrType& lst = GetMenuRegion(mn).List();
     for( MenuRegion::Itr itr = lst.begin(), end = lst.end(); itr != end; ++itr )
@@ -253,35 +341,56 @@ static bool ScriptMenu(xmlpp::Element* menus_node, Menu root_menu, Menu mn, int
         {
             std::string targ_str;    
             if( HasButtonLink(*m_obj, targ_str) )
-            {
-                xmlpp::Element* node = pgc_node->add_child("button");
-                node->add_child_text(targ_str);
-            }
+                AddChildWithText(pgc_node, "button", targ_str);
         }
     return true;
 }
 
-typedef boost::function<std::string(VideoItem)> TitlePostCommand;
-
-static std::string JumpNextTitle(VideoItem vi, int titles_cnt)
+static std::string SrcFilename(VideoItem vi)
 {
-    int cur_num = GetAuthorNumber(*vi);
-    ASSERT( (cur_num > 0) && (cur_num <= titles_cnt) );
-
-    int next_num = (cur_num == titles_cnt) ? 1 : cur_num + 1;
-    return "jump title " + boost::lexical_cast<std::string>(next_num) + ";";
+    return GetFilename(*vi);
 }
 
-static std::string AutoPostCmd(const std::string& jnt_cmd)
+static std::string PrefixCnvPath(VideoItem vi, const std::string& out_dir)
 {
-    return MenusCnt() ? std::string("call menu entry root;") : jnt_cmd ;
+    std::string dst_fname = boost::format("%1%.%2%") % GetAuthorNumber(vi) 
+        % fs::path(SrcFilename(vi)).leaf() % bf::stop;
+    return AppendPath(out_dir, dst_fname);
 }
 
-static bool ScriptTitle(xmlpp::Element* ts_node, VideoItem vi, int titles_cnt)
+// путь для скрипта dvdauthor
+static std::string DVDCompliantFilename(VideoItem vi, const std::string& out_dir)
+{
+    std::string dst_fname = SrcFilename(vi);
+    if( RequireTranscoding(vi) )
+        dst_fname = PrefixCnvPath(vi, out_dir) + ".mpg";
+    return dst_fname;
+}
+
+static bool IsToAddSubtitles(VideoItem vi)
+{
+    return !vi->subDat.pth.empty();
+}
+
+static std::string DVDFilename(VideoItem vi, const std::string& out_dir)
+{
+    std::string dst_fname = DVDCompliantFilename(vi, out_dir);
+    if( IsToAddSubtitles(vi) )
+        dst_fname = PrefixCnvPath(vi, out_dir) + ".sub.mpg";
+    return dst_fname;
+}
+
+static bool ScriptTitle(xmlpp::Element* ts_node, VideoItem vi, const std::string& out_dir)
 {
     xmlpp::Element* pgc_node = ts_node->add_child("pgc");
+    if( IsToAddSubtitles(vi) )
+    {
+        int sub_num = vi->subDat.defShow ? 64 : 0 ;
+        AddChildWithText(pgc_node, "pre", boost::format("subtitle = %1%;") % sub_num % bf::stop);
+    }
+
     xmlpp::Element* vob_node = pgc_node->add_child("vob");
-    vob_node->set_attribute("file", GetFilename(*vi));
+    vob_node->set_attribute("file", DVDFilename(vi, out_dir));
     // список глав
     bool is_empty = true;
     std::string chapters;
@@ -297,29 +406,7 @@ static bool ScriptTitle(xmlpp::Element* ts_node, VideoItem vi, int titles_cnt)
     if( !is_empty )
         vob_node->set_attribute("chapters", chapters);
 
-    xmlpp::Element* post_node = pgc_node->add_child("post");
-
-    std::string jnt_cmd = JumpNextTitle(vi, titles_cnt);
-    // чаще всего будет эта команда
-    std::string post_cmd = AutoPostCmd(jnt_cmd);
-    PostAction& pa = vi->PAction();
-    switch( pa.paTyp )
-    {
-    case patAUTO:
-        break;
-    case patNEXT_TITLE:
-        post_cmd = jnt_cmd;
-        break;
-    case patEXP_LINK:
-        if( pa.paLink )
-            post_cmd = MakeButtonJump(pa.paLink, true);
-        break;
-    default:
-        //ASSERT_RTL(0);
-        break; // не вылетаем на плохих проектах
-    }
-    post_node->add_child_text(post_cmd);
-
+    AddPostCmd(pgc_node, vi);
     return true;
 }
 
@@ -330,10 +417,21 @@ static Menu GetFirstMenu()
     return (menus_be.first != menus_be.second) ? GetMenu(mn_store, menus_be.first) : Menu();
 }
 
+//static bool _GetFirstVideo(VideoItem& res, VideoItem vi, int)
+//{
+//    res = vi;
+//    return false;
+//}
+
 VideoItem GetFirstVideo()
 {
     VideoItem first_vi;
-    ForeachVideo(bl::bind(&_GetFirstVideo, boost::ref(first_vi), bl::_1, bl::_2));
+    //ForeachVideo(bb::bind(&_GetFirstVideo, boost::ref(first_vi), _1, _2));
+    boost_foreach( VideoItem vi, AllVideos() )
+    {
+        first_vi = vi;
+        break;
+    }
     return first_vi;
 }
 
@@ -343,7 +441,8 @@ bool Is4_3(VideoItem first_vi)
     if( first_vi )
     {
         Point asp = Project::CalcAspectSize(*first_vi);
-        if( asp == Point(16, 9) )
+        // с погрешностью 0.1
+        if( (double)asp.x/asp.y > (4 + 0.1)/3. )
             is_4_3 = false;
     }
     return is_4_3;
@@ -365,9 +464,29 @@ bool IsMenuToBe4_3()
     return is_menu_4_3;
 }
 
+//bool IndexForAuthoring(VideoItem vi, int idx)
+//{
+//    GetAuthorNumber(vi) = idx+1; // индекс от 1
+//    return true;
+//}
+
+// 2 цели:
+// - в первую очередь, для скрипта dvdauthor
+// - для нумерации при транскодировании
+void IndexVideosForAuthoring()
+{
+    int idx = 1; // индекс от 1
+    boost_foreach( VideoItem vi, AllVideos() )
+        GetAuthorNumber(vi) = idx++;
+}
+
 void GenerateDVDAuthorScript(const std::string& out_dir)
 {
-    int titles_cnt = IndexVideosForAuthoring();
+    // индексируем видео в titles
+    //IndexVideosForAuthoring();
+    //ForeachVideo(bb::bind(&IndexForAuthoring, _1, _2));
+    //IndexVideosForAuthoring();
+
     ADatabase& db = AData();
     AStores&   as = GetAStores();
 
@@ -405,9 +524,9 @@ void GenerateDVDAuthorScript(const std::string& out_dir)
     {
         xmlpp::Element* vmgm_node = root_node->add_child("vmgm");
         vmgm_node->add_child_comment("First Play");
-        xmlpp::Element* node = vmgm_node->add_child("fpc");
-        node->add_child_text("jump menu entry title;");
-        node = vmgm_node->add_child("menus");
+        AddChildWithText(vmgm_node, "fpc", "jump menu entry title;");
+
+        xmlpp::Element* node = vmgm_node->add_child("menus");
         // из-за того, что в VMG мы не устанавливаем ни одно видео, то
         // надо явно инициализировать атрибуты (см. функцию BuildAVInfo()
         // в dvdauthor):
@@ -425,11 +544,13 @@ void GenerateDVDAuthorScript(const std::string& out_dir)
         // кнопка "Title" = кнопка "Menu"
         xmlpp::Element* title_entry = node->add_child("pgc");
         title_entry->set_attribute("entry", "title");
-        std::string init_cmd = "jump " + MakeFPTarget(fp) + ";";
+        // g1 - номер текущего меню
+        // g2 - признак Play All
+        std::string init_cmd = "g2 = 0; jump " + MakeFPTarget(fp) + ";";
         if( root_menu )
             // если есть меню (вообще есть), то инициализируем первое
             init_cmd = boost::format("g1 = %1%; %2%") % GetMenuANum(root_menu) % init_cmd % bf::stop;
-        title_entry->add_child("pre")->add_child_text(init_cmd);
+        AddChildWithText(title_entry, "pre", init_cmd);
     }
     // основная часть
     {
@@ -437,26 +558,31 @@ void GenerateDVDAuthorScript(const std::string& out_dir)
         // * меню
         xmlpp::Element* menus_node = tts_node->add_child("menus");
         AddVideoTag(menus_node, IsMenuToBe4_3());
-        ForeachMenu(bl::bind(&ScriptMenu, menus_node, root_menu, bl::_1, bl::_2));
+        ForeachMenu(bb::bind(&ScriptMenu, menus_node, root_menu, _1, _2));
         // * список разделов (titles)
         xmlpp::Element* ts_node = tts_node->add_child("titles");
         AddVideoTag(ts_node, Is4_3(first_vi));
 
-        ForeachVideo(bl::bind(&ScriptTitle, ts_node, bl::_1, titles_cnt));
+        ForeachVideo(bb::bind(&ScriptTitle, ts_node, _1, out_dir));
     }
-    doc.write_to_file_formatted(AppendPath(out_dir, "DVDAuthor.xml"));
+    SaveFormattedUTF8Xml(doc, AppendPath(out_dir, "DVDAuthor.xml"));
 }
 
-static bool AuthorClearVideo(VideoItem vi, int)
+static bool AuthorClearVideo(VideoItem vi)
 {
     vi->Clear(AUTHOR_TAG);
     return true;
 }
 
-static bool AuthorClearMenu(Menu mn, int)
+void ClearTaggedData(Menu mn, const char* tag)
 {
-    mn->Clear(AUTHOR_TAG);
-    Editor::ClearLocalData(GetMenuRegion(mn), AUTHOR_TAG);
+    mn->Clear(tag);
+    Editor::ClearLocalData(GetMenuRegion(mn), tag);
+}
+
+static bool AuthorClearMenu(Menu mn)
+{
+    ClearTaggedData(mn, AUTHOR_TAG);
     return true;
 }
 
@@ -503,13 +629,177 @@ struct RestrictGetCanvasBuf
 
     void Do(bool is_on)
     {
-        using namespace boost;
-        if( CheckAuthorMode::Flag )
-            ForeachMenu(lambda::bind(&RestrictGetCanvasBufMenu, lambda::_1, is_on));
+        if( Execution::ConsoleMode::Flag )
+            ForeachMenu(bb::bind(&RestrictGetCanvasBufMenu, _1, is_on));
     }
 };
 
-bool AuthorDVD(const std::string& out_dir)
+static void CheckHomeSpumuxFont()
+{
+    char* home_dir = getenv("HOME");
+    if( !home_dir )
+        Error("Cannot find HOME directory!");
+
+    fs::path font_path = fs::path(home_dir)/"/.spumux/FreeSans.ttf";
+    if( !fs::exists(font_path) )
+    {
+        std::string err_str;
+        if( !CreateDirs(font_path.branch_path(), err_str) )
+            Error(err_str.c_str());
+        fs::copy_file(DataDirPath("copy-n-paste/FreeSans.ttf"), font_path);
+    }
+}
+
+std::string FFmpegToDVDTranscode(const std::string& src_fname, const std::string& dst_fname,
+                                 const AutoDVDTransData& atd, bool is_pal, const DVDTransData& td)
+{
+    return boost::format("ffmpeg -i %1% %2%") % FilenameForCmd(src_fname) 
+        % FFmpegToDVDArgs(dst_fname, atd, is_pal, td) % bf::stop;
+}
+
+// ffmpeg выводит размер первого создаваемого файла каждые полсекунды,
+// см. print_report() (при verbose=1, по умолчанию)
+// Формат: "size=%8.0fkB"
+re::pattern FFmpegSizePat( "size= *"RG_NUM"kB"); 
+
+static void OnTranscodePrintParse(const char* dat, int sz, const io::pos trans_total,
+                                  const io::pos trans_done, const io::pos trans_val)
+{
+    re::match_results what;
+    if( re::search(std::string(dat, sz), what, FFmpegSizePat) )
+    {
+        // :KLUDGE: вроде и нецелые могут быть, но precision=0
+        // как-то не в тему
+        io::pos sz = 1024 * boost::lexical_cast<io::pos>(what.str(1));
+        double per = (std::min(sz, trans_val) + trans_done)/double(trans_total);
+        Author::SetStageProgress(per);
+    }
+}
+
+static void AuthorImpl(const std::string& out_dir)
+{
+    AuthorSectionInfo((str::stream() << "Build DVD-Video in folder: " << out_dir).str());
+    IteratePendingEvents();
+
+    IndexVideosForAuthoring();
+
+    // * транскодирование
+    io::pos trans_done = 0, trans_total = ProjectStat().transSum;
+    Author::SetStage(Author::stTRANSCODE);
+    boost_foreach( VideoItem vi, AllTransVideos() )
+    {
+        CheckFFDVDEncoding();
+        std::string src_fname = SrcFilename(vi);
+        std::string dst_fname = DVDCompliantFilename(vi, out_dir);
+        // :TRICKY: отдельные видеофайлы подгоняем под DVD-формат исходя из них самих
+        // (а не под одну "гребенку" проекта), несмотря на то, что запись в один VTS видео
+        // разных форматов вроде как противоречит спекам. Причина: вроде и так работает (на
+        // доступных мне железных и софтовых плейерах), а если будут проблемы - лучше мультиформат
+        // реализовать
+        RTCache& rtc = GetRTC(vi);
+        AutoDVDTransData atd(Is4_3(vi));
+        atd.audioNum  = OutAudioNum(rtc.audioNum);
+        atd.srcAspect = rtc.dar;
+        DVDTransData td = GetRealTransData(vi);
+
+        std::string ffmpeg_cmd = FFmpegToDVDTranscode(src_fname, dst_fname, atd, IsPALProject(), td);
+        io::pos trans_val      = CalcTransSize(rtc, td.vRate);
+        RunFFmpegCmd(ffmpeg_cmd, bb::bind(&OnTranscodePrintParse, _1, _2, trans_total, trans_done, trans_val));
+
+        trans_done += trans_val;
+    }
+
+    // * субтитры
+    boost_foreach( VideoItem vi, AllVideos() )
+        if( IsToAddSubtitles(vi) )
+        {
+            xmlpp::Document doc;
+            xmlpp::Element* ts = doc.create_root_node("subpictures")->add_child("stream")->add_child("textsub");
+
+            SubtitleData& dat = vi->subDat;
+            ts->set_attribute("filename", dat.pth);
+            ts->set_attribute("font", "FreeSans.ttf");
+            //ts->set_attribute("force", "no");
+
+            // :KLUDGE:
+            // софтовые плейеры Xine, Vlc и Totem хотят размеров, соответ. 
+            // самому видео (Totem вообще падает иначе), а вот мой железячный плейер 
+            // не любит ничего кроме 720xfull и 28.0,- 
+            // решил в пользу софтовых ради удобства разработки, но лучше бы
+            // разобраться (хорошо, что не 720xfull - редкость)
+            DVDDims dd = RequireTranscoding(vi) ? GetRealTransData(vi).dd : CalcDimsAuto(vi);
+            // чуть меньше делаем размеры
+            Point movie_sz = DVDDimension(dd) - Point(2, 2);
+            ts->set_attribute("movie-width",  Int2Str(movie_sz.x));
+            ts->set_attribute("movie-height", Int2Str(movie_sz.y));
+            double font_sz = 28.;
+            switch( dd )
+            {
+            case dvd352s:
+                font_sz = 14.;
+                break;
+            case dvd352:
+                font_sz = 21.;
+                break;
+            default:
+                break;
+            }
+            ts->set_attribute("fontsize", Double2Str(font_sz));
+
+            if( !dat.encoding.empty() )
+                ts->set_attribute("characterset", dat.encoding);
+
+            ts->set_attribute("horizontal-alignment", "center");
+            ts->set_attribute("vertical-alignment",   "bottom");
+
+            std::string xml_fname = PrefixCnvPath(vi, out_dir) + ".textsub.xml";
+            SaveFormattedUTF8Xml(doc, xml_fname);
+            
+            CheckHomeSpumuxFont();
+            std::string spumux_cmd = boost::format("spumux -m dvd %1% < %2% > %3%") 
+                % FilenameForCmd(xml_fname) % FilenameForCmd(DVDCompliantFilename(vi, out_dir))
+                % FilenameForCmd(DVDFilename(vi, out_dir)) % bf::stop;
+            RunExtCmd(spumux_cmd, "spumux");
+        }
+
+    Author::ExecState& es = Author::GetES();
+    str::stream& settings = es.settings;
+    settings.str("# coding: utf-8\n\n");
+    // * рендерим меню и их скрипты
+    AuthorMenus(out_dir);
+
+    // *
+    GenerateDVDAuthorScript(out_dir);
+
+    // * вспомогательные файлы
+    const char* fnames[] = { "SConstruct", "ADVD.py", "SConsTwin.py", "README" };
+    for( int i=0; i<(int)ARR_SIZE(fnames); i++ )
+        CopyRootFile(fnames[i], out_dir);
+    // опции
+    str::stream scons_options;
+    Author::FillSconsOptions(scons_options, true);
+    // ASettings.py
+    io::stream settings_strm(AppendPath(out_dir, "ASettings.py").c_str(), iof::out);
+    settings_strm << settings.rdbuf();
+    settings_strm.close();
+
+    // *
+    if( !Execution::ConsoleMode::Flag )
+    {
+        if( es.mode != Author::modRENDERING )
+        {
+            Author::BuildDvdOF of;
+            Author::ExecuteSconsCmd(out_dir, of, es.mode, scons_options);
+        }
+    }
+    else
+    {
+        ////int pid = gnome_execute_shell(out_dir.c_str(), "scons totem");
+        //int pid = Spawn(out_dir.c_str(), "scons totem");
+    }
+}
+
+std::string AuthorDVD(const std::string& out_dir)
 {
     ASSERT( fs::is_directory(out_dir) );
     ASSERT( fs::is_empty_directory(out_dir) );
@@ -518,67 +808,19 @@ bool AuthorDVD(const std::string& out_dir)
     // для тестов авторинга - не даем пользоваться GetCanvasBuf(),
     // так как он для интерактива
     RestrictGetCanvasBuf rgcb;
-    bool res = true;
-    try
-    {
-        AuthorSectionInfo((str::stream() << "Build DVD-Video in folder: " << out_dir).str());
-        IteratePendingEvents();
-
-        Author::ExecState& es = Author::GetES();
-        str::stream& settings = es.settings;
-        settings.str("# coding: utf-8\n\n");
-        // * рендерим меню и их скрипты
-        AuthorMenus(out_dir);
-
-        // *
-        GenerateDVDAuthorScript(out_dir);
-
-        // * вспомогательные файлы
-        const char* fnames[] = { "SConstruct", "ADVD.py", "SConsTwin.py", "README" };
-        for( int i=0; i<(int)ARR_SIZE(fnames); i++ )
-            CopyRootFile(fnames[i], out_dir);
-        // опции
-        str::stream scons_options;
-        Author::FillSconsOptions(scons_options, true);
-        // ASettings.py
-        io::stream settings_strm(AppendPath(out_dir, "ASettings.py").c_str(), iof::out);
-        settings_strm << settings.rdbuf();
-        settings_strm.close();
-
-        // *
-        if( !CheckAuthorMode::Flag )
-        {
-            if( es.mode != Author::modRENDERING )
-            {
-                Author::BuildDvdOF of;
-                ExitData ed = Author::ExecuteSconsCmd(out_dir, of, es.mode, scons_options);
-    
-                if( !ed.IsGood() )
-                    throw std::runtime_error(BF_("external command failure: %1%") % ExitDescription(ed) % bf::stop);
-            }
-        }
-        else
-        {
-            ////int pid = gnome_execute_shell(out_dir.c_str(), "scons totem");
-            //int pid = Spawn(out_dir.c_str(), "scons totem");
-            //ASSERT(pid > 0);
-        }
-    }
-    catch (const std::exception& err)
-    {
-        Author::GetES().exitDesc = err.what();
-        Author::Info(std::string("ERR: ") + err.what(), false);
-        Author::Info("Stop authoring!\n", false);
-        res = false;
-    }
+    std::string res = Author::SafeCall(bb::bind(&AuthorImpl, out_dir));
 
     // очищаем все авторинговые данные
-    using namespace boost;
-    ForeachVideo(lambda::bind(&AuthorClearVideo, lambda::_1, lambda::_2));
-    ForeachMenu(lambda::bind(&AuthorClearMenu, lambda::_1, lambda::_2));
+    ForeachVideo(bb::bind(&AuthorClearVideo, _1));
+    ForeachMenu(bb::bind(&AuthorClearMenu, _1));
 
-    if( res )
+    if( Author::IsGood(res) )
         AuthorSectionInfo("Authoring is successfully done.");
+    else
+    {
+        Author::Info(std::string("ERR: ") + res, false);
+        Author::Info("Stop authoring!\n", false);
+    }
     return res;
 }
 
@@ -588,7 +830,7 @@ namespace Author {
 
 static void AppendToLog(const std::string& txt)
 {
-    if( Project::CheckAuthorMode::Flag )
+    if( Execution::ConsoleMode::Flag )
         LOG_INF << txt << io::endl;
     else
         AppendText(GetES().detailsView, txt + "\n");
@@ -604,11 +846,6 @@ void Info(const std::string& str, bool add_info_sign)
     AppendToLog((add_info_sign ? "INFO: " : "") + str);
 }
 
-void Error(const std::string& str)
-{
-    throw std::runtime_error(str);
-}
-
 std::string SconsTarget(Mode mode)
 {
     std::string res;
@@ -653,7 +890,7 @@ void FillSconsOptions(str::stream& scons_options, bool fill_def)
 {
     std::string def_dvd_label, def_drive;
     double def_speed = 0;
-    if( !Project::CheckAuthorMode::Flag )
+    if( !Execution::ConsoleMode::Flag )
     {     
         BurnData& bd = GetBD();
         def_dvd_label = bd.Label().get_text();
@@ -665,11 +902,63 @@ void FillSconsOptions(str::stream& scons_options, bool fill_def)
     AddSconsOptions(scons_options, true, def_dvd_label, def_drive, def_speed);
 }
 
-ExitData ExecuteSconsCmd(const std::string& out_dir, OutputFilter& of, 
-                         Mode mod, const str::stream& scons_options)
+void CheckAbortByUser()
+{
+    if( Author::GetES().eDat.userAbort )
+        throw std::runtime_error("User Abortion"); // строка реально не нужна - сработает userAbort
+}
+
+ExitData AsyncCall(const char* dir, const char* cmd, const ReadReadyFnr& fnr)
+{
+    ExitData ed = ExecuteAsync(dir, cmd, fnr, &GetES().eDat.pid);
+    CheckAbortByUser();
+
+    return ed;
+}
+
+ReadReadyFnr OF2RRF(OutputFilter& of)
+{
+    return bb::bind(&Author::OutputFilter::OnGetLine, &of, _1, _2, _3);
+}
+
+void Error(const std::string& str)
+{
+    throw std::runtime_error(str);
+}
+
+void ApplicationError(const char* app_name, const std::string& reason)
+{
+    Author::Error(BF_("%1% failure: %2%") % app_name % reason % bf::stop);
+}
+
+void ApplicationError(const char* app_name, const ExitData& ed)
+{
+    ApplicationError(app_name, ExitDescription(ed));
+}
+
+void ExecuteSconsCmd(const std::string& out_dir, OutputFilter& of, 
+                     Mode mod, const str::stream& scons_options)
 {
     std::string cmd = "scons" + scons_options.str() + " " + SconsTarget(mod);
-    return ExecuteAsync(out_dir.c_str(), cmd.c_str(), of, &GetES().eDat.pid);
+    ExitData ed = AsyncCall(out_dir.c_str(), cmd.c_str(), OF2RRF(of));
+    if( !ed.IsGood() )
+        //ApplicationError("", ed);
+        Error(BF_("external command failure: %1%") % ExitDescription(ed) % bf::stop);
+}
+
+std::string SafeCall(const ActionFunctor& fnr)
+{
+    std::string result;
+    try
+    {
+        fnr();
+    }
+    catch (const std::exception& err)
+    {
+        result = err.what();
+        ASSERT( !result.empty() );
+    }
+    return result;
 }
 
 } // namespace Author
