@@ -22,257 +22,32 @@
 #include <mgui/_pc_.h>
 
 #include "mconstructor.h"
-#include "menu-actions.h"
-#include "menu-render.h"
-#include "mb-actions.h"
-#include <mgui/text_obj.h> // CalcAbsSizes()
+#include "serialize.h"
 
-#include <mgui/win_utils.h>
-#include <mgui/sdk/window.h>
-#include <mgui/sdk/packing.h>
-#include <mgui/sdk/menu.h>
-#include <mgui/sdk/widget.h>
+#include "mb-actions.h"
+#include "menu-actions.h"
+#include "add.h"
+
+#include <mgui/editor/kit.h> // new Editor::Kit()
+#include <mgui/text_obj.h> // CalcAbsSizes()
 #include <mgui/trackwindow.h>
-#include <mgui/dialog.h> // ChooseFileSaveTo()
-#include <mgui/img-factory.h> // GetFactoryImage()
-#include <mgui/author/script.h> // VideoSizeSum()
+#include <mgui/author/script.h> // ForeachVideo()
 #include <mgui/author/output.h> // PackOutput()
 
-#include <mgui/mux.h>
+#include <mgui/sdk/window.h>
+#include <mgui/sdk/menu.h>
+
 #include <mgui/prefs.h>
+#include <mgui/mux.h>
 #include <mgui/gettext.h>
+
 #include <mgui/project/handler.h> // RegisterHook()
+
 #include <mlib/sigc.h>
-#include <mlib/filesystem.h>
 
 #include <gtk/gtkaboutdialog.h>
 #include <gtk/gtklinkbutton.h> // gtk_button_set_relief
 
-namespace Project
-{
-
-void PackMediasWindow(Gtk::Container& contr, RefPtr<MediaStore> ms, MediasWindowPacker pack_fnr)
-{
-    MediaBrowser& brw = NewManaged<MediaBrowser>(ms);
-    pack_fnr(contr, brw);
-}
-
-void PackFullMBrowser(Gtk::Container& contr, MediaBrowser& brw)
-{
-    using namespace boost;
-    PackTrackWindow(contr, lambda::bind(&PackMBWindow, lambda::_1, lambda::_2, lambda::_3, boost::ref(brw)));
-}
-
-void PackMediasWindow(Gtk::Container& contr, RefPtr<MediaStore> ms)
-{
-    PackMediasWindow(contr, ms, &Project::PackFullMBrowser);
-}
-
-void SetTabName(Gtk::Notebook& nbook, const std::string& name, int pos)
-{
-    Gtk::Label& lbl = NewMarkupLabel("<span weight = \"bold\" style  = \"italic\">"
-                                     + name + "</span>", true);
-
-    // попытка сделать вертикальные закладки читабельными
-    //switch( nbook.get_tab_pos() )
-    //{
-    //case Gtk::POS_LEFT:
-    //case Gtk::POS_RIGHT:
-    //    lbl.set_angle(+90.);
-    //    {
-    //        RefPtr<Pango::Layout> lay = lbl.get_layout();
-    //        //lay->set_spacing(0);
-    //        PangoContext* cnxt = lay->get_context()->gobj();
-    //        pango_context_set_base_gravity(cnxt, PANGO_GRAVITY_EAST);
-    //        pango_context_set_gravity_hint(cnxt, PANGO_GRAVITY_HINT_STRONG);
-    //    }
-    //    break;
-    //default:
-    //    ;
-    //}
-
-    nbook.set_tab_label(*nbook.get_nth_page(pos), lbl);
-}
-
-static void SaveProjectData(RefPtr<MenuStore> mn_store, RefPtr<MediaStore> md_store)
-{
-    ADatabase& db = AData();
-    // * сохраняемся
-    for( MediaStore::iterator itr = md_store->children().begin(), end = md_store->children().end();
-         itr != end; ++itr )
-        db.GetML().Insert(md_store->GetMedia(itr));
-    for( MenuStore::iterator  itr = mn_store->children().begin(), end = mn_store->children().end();
-         itr != end; ++itr )
-    {
-        Menu mn = GetMenu(mn_store, itr);
-        Project::SaveMenu(mn);
-        db.GetMN().Insert(mn);
-    }
-    db.SetOut(false);
-    db.Save();
-}
-
-static std::string MakeProjectTitle(bool with_path_breakdown = false)
-{
-    ADatabase& db = AData();
-    if( !db.IsProjectSet() )
-        return "untitled.xml";
-
-    fs::path full_path(db.GetProjectFName());
-    std::string res_str = full_path.leaf();
-    if( with_path_breakdown )
-        res_str += " (" + full_path.branch_path().string() + ")";
-    return res_str;
-}
-
-static bool SaveProjectAs(Gtk::Widget& for_wdg) 
-{
-    bool res = false;
-    std::string fname = MakeProjectTitle();
-    if( ChooseFileSaveTo(fname, _("Save Project As..."), for_wdg) )
-    {
-        fname = Project::ConvertPathToUtf8(fname);
-
-        ASSERT( !fname.empty() );
-        AData().SetProjectFName(fname);
-        res = true;
-    }
-
-    return res;
-}
-
-AStores& GetAStores()
-{
-    return AData().GetData<AStores>();
-}
-
-static void LoadProjectInteractive(const std::string& prj_file_name)
-{
-    ADatabase& db = AData();
-    bool res = true;
-    std::string err_str;
-    try
-    {
-        void DbSerializeProjectImpl(Archieve& ar);
-        db.LoadWithFnr(prj_file_name, DbSerializeProjectImpl);
-    }
-    catch (const std::exception& err)
-    {
-        res = false;
-        err_str = err.what();
-    }
-    if( !res )
-    {
-        // мягкая очистка
-        db.Clear(false);
-        db.ClearSettings();
-        MessageBox(BF_("Cant open project file \"%1%\"") % prj_file_name % bf::stop, Gtk::MESSAGE_ERROR, 
-                   Gtk::BUTTONS_OK, err_str);
-    }
-}
-
-static bool ClearEndAction(VideoItem vi, MediaItem mi)
-{
-    PostAction& pa = vi->PAction();
-    if( pa.paLink == mi )
-    {
-        pa.paLink = 0;
-        pa.paTyp  = patAUTO;
-    }
-    return true;
-}
-
-static void OnDeleteEndAction(MediaItem mi, const char* action)
-{
-    if( mi && (strcmp("OnDelete", action) == 0) )
-        ForeachVideo(bl::bind(ClearEndAction, bl::_1, mi));
-}
-
-// создать списки медиа и меню
-static AStores& InitAStores()
-{
-    RefPtr<MediaStore> md_store = CreateEmptyMediaStore();
-    RefPtr<MenuStore>  mn_store = CreateEmptyMenuStore();
-
-    AStores& as = GetAStores();
-    as.mdStore  = md_store;
-    as.mnStore  = mn_store;
-
-    // для ForeachVideo требуется готовый as.mdStore
-    RegisterHook(&OnDeleteEndAction);
-    return as;
-}
-
-static void SetAppTitle(ConstructorApp& app, bool clear_change_flag = true);
-static void UpdateDVDSize(SizeBar& sz_bar);
-
-static void LoadApp(ConstructorApp& app, const std::string& fname)
-{
-    LoadProjectInteractive(ConvertPathToUtf8(fname));
-
-    AStores& as = GetAStores();
-    ADatabase& db = AData();
-
-    PublishMediaStore(as.mdStore);
-    PublishMenuStore(as.mnStore, db.GetMN());
-    UpdateDVDSize(app.SB());
-
-    db.SetOut(true);
-    SetAppTitle(app);
-}
-
-AStores& InitAndLoadPrj(const std::string& fname)
-{
-    AStores& as = InitAStores();
-
-    ADatabase& db = AData();
-    db.Load(ConvertPathToUtf8(fname));
-
-    PublishMediaStore(as.mdStore);
-    PublishMenuStore(as.mnStore, db.GetMN());
-
-    db.SetOut(true);
-    return as;
-}
-
-static Rect GetAllocation(Gtk::Widget& wdg)
-{
-    return MakeRect(*wdg.get_allocation().gobj());
-}
-
-static bool EraseTabLineOnExpose(Gtk::Notebook& nbook, GdkEventExpose* event, bool erase_down,
-				 Gtk::Notebook* book_tabs)
-{
-    Rect plc = GetAllocation(nbook);
-    if( !plc.IsNull() )
-    {
-        plc.lft += 1;
-        if( erase_down )
-    	{
-    	    plc.rgt -= 1;
-    	    plc.top = plc.btm-1;
-    	}
-        else
-    	{
-            // стираем верхнюю линию рамки только до окончания верхних табов
-    	    plc.rgt = std::min(plc.rgt, GetAllocation(*book_tabs).rgt)-1;
-    	    plc.btm = plc.top+1;
-    	}
-        if( plc.Intersects(MakeRect(event->area)) )
-        {
-            Cairo::RefPtr<Cairo::Context> cr = nbook.get_window()->create_cairo_context();
-            cr->set_line_width(erase_down ? 1.0 : 2.0);
-            CR::SetColor(cr, GetBGColor(nbook));
-
-            double y = plc.top + (erase_down ? H_P : 1);
-            cr->move_to(plc.lft, y);
-            cr->line_to(plc.rgt, y);
-            cr->stroke();
-        }
-    }
-    return true;
-}
-
 //
 // COPY_N_PASTE_ETALON из go-file.c, проект Gnumeric, http://projects.gnome.org/gnumeric/
 //
@@ -385,6 +160,118 @@ go_url_show(GtkAboutDialog*, const gchar* url, gpointer)
 // COPY_N_PASTE_ETALON_END из go-file.c, проект Gnumeric, http://projects.gnome.org/gnumeric/
 //
 
+void GoUrl(const gchar* url)
+{
+    go_url_show(0, url, 0);
+}
+
+namespace Project
+{
+
+void PackMediasWindow(Gtk::Container& contr, RefPtr<MediaStore> ms, MediasWindowPacker pack_fnr)
+{
+    MediaBrowser& brw = NewManaged<MediaBrowser>(ms);
+    pack_fnr(contr, brw);
+}
+
+void PackFullMBrowser(Gtk::Container& contr, MediaBrowser& brw)
+{
+    PackTrackWindow(contr, bb::bind(&PackMBWindow, _1, _2, _3, boost::ref(brw)));
+}
+
+void PackMediasWindow(Gtk::Container& contr, RefPtr<MediaStore> ms)
+{
+    PackMediasWindow(contr, ms, &Project::PackFullMBrowser);
+}
+
+AStores& GetAStores()
+{
+    return AData().GetData<AStores>();
+}
+
+static void ClearPA(PostAction& pa, MediaItem del_mi)
+{
+    if( pa.paLink == del_mi )
+    {
+        pa.paLink = 0;
+        pa.paTyp  = patAUTO;
+    }
+}
+
+static bool ClearEndAction(VideoItem vi, MediaItem mi)
+{
+    ClearPA(vi->PAction(), mi);
+    return true;
+}
+
+static bool ClearEndActionM(Menu mn, MediaItem mi)
+{
+    ClearPA(mn->MtnData().pAct, mi);
+    return true;
+}
+
+static void OnDeleteEndAction(MediaItem mi, const char* action)
+{
+    if( mi && (strcmp("OnDelete", action) == 0) )
+    {
+        ForeachVideo(bb::bind(ClearEndAction, _1, mi));
+        ForeachMenu(bb::bind(ClearEndActionM, _1, mi));
+    }
+}
+
+// создать списки медиа и меню
+AStores& InitAStores()
+{
+    RefPtr<MediaStore> md_store = CreateEmptyMediaStore();
+    RefPtr<MenuStore>  mn_store = CreateEmptyMenuStore();
+
+    AStores& as = GetAStores();
+    as.mdStore  = md_store;
+    as.mnStore  = mn_store;
+
+    // для ForeachVideo требуется готовый as.mdStore
+    RegisterHook(&OnDeleteEndAction);
+    return as;
+}
+
+Rect GetAllocation(Gtk::Widget& wdg)
+{
+    return MakeRect(*wdg.get_allocation().gobj());
+}
+
+static bool EraseTabLineOnExpose(Gtk::Notebook& nbook, GdkEventExpose* event, bool erase_down,
+				 Gtk::Notebook* book_tabs)
+{
+    Rect plc = GetAllocation(nbook);
+    if( !plc.IsNull() )
+    {
+        plc.lft += 1;
+        if( erase_down )
+    	{
+    	    plc.rgt -= 1;
+    	    plc.top = plc.btm-1;
+    	}
+        else
+    	{
+            // стираем верхнюю линию рамки только до окончания верхних табов
+    	    plc.rgt = std::min(plc.rgt, GetAllocation(*book_tabs).rgt)-1;
+    	    plc.btm = plc.top+1;
+    	}
+        if( plc.Intersects(MakeRect(event->area)) )
+        {
+            Cairo::RefPtr<Cairo::Context> cr = nbook.get_window()->create_cairo_context();
+            cr->set_line_width(erase_down ? 1.0 : 2.0);
+            CR::SetColor(cr, GetBGColor(nbook));
+
+            double y = plc.top + (erase_down ? H_P : 1);
+            cr->move_to(plc.lft, y);
+            cr->line_to(plc.rgt, y);
+            cr->stroke();
+        }
+    }
+    return true;
+}
+
 static void HookUpURLs()
 {
     static bool is_init = false;
@@ -410,7 +297,13 @@ static void OnCloseAboutDlg(Gtk::AboutDialog* dlg, int reps)
         dlg->hide();
 }
 
-static void OnDlgAbout(Gtk::Window& win)
+static void OnlineHelp()
+{
+    //GoUrl("http://www.bombono.org/Documentation");
+    GoUrl("http://www.bombono.org/Bombono_Tutorial");
+}
+
+static void OnDlgAbout()
 {
     // :TODO: включить кнопку лицензии пока нельзя из ограничения окна About на размер
     //const char* license =
@@ -455,22 +348,22 @@ static void OnDlgAbout(Gtk::Window& win)
     {
         dlg = new Gtk::AboutDialog;
         dlg->Gtk::Widget::set_name("AboutBombonoDVD");
-        dlg->set_transient_for(win);
+        dlg->set_transient_for(Application().win);
     
         dlg->set_name(APROJECT_NAME);
         dlg->set_version(APROJECT_VERSION);
-        dlg->set_copyright("Copyright \xc2\xa9 2007-2009 Ilya Murav'jov");
+        dlg->set_copyright("Copyright \xc2\xa9 2007-2010 Ilya Murav'jov");
         //dlg->set_license(license);
         dlg->set_website("http://www.bombono.org");
         dlg->set_comments(_("Bombono DVD is a DVD authoring program with nice and clean GUI"));
-        dlg->set_logo(Gdk::Pixbuf::create_from_file(AppendPath(GetDataDir(), "about-front.png")));
+        dlg->set_logo(DataDirImage("about-front.png"));
         //dlg.set_authors(authors);
         //dlg.set_documenters(documenters);
         //dlg.set_decorated(false);
         dlg->set_translator_credits(_("translator-credits"));
     
         ForAllWidgets(static_cast<Gtk::Widget&>(*dlg).gobj(), SetNoButtonRelief);
-        dlg->signal_response().connect(bl::bind(OnCloseAboutDlg, dlg.get(), bl::_1));
+        dlg->signal_response().connect(bb::bind(OnCloseAboutDlg, dlg.get(), _1));
     }
 
     //dlg.run();
@@ -516,13 +409,14 @@ static void OnGoItemToggled(Gtk::RadioMenuItem& itm, ConstructorApp& app)
 static Gtk::RadioMenuItem& CreateGoMenuItem(Gtk::RadioButtonGroup& grp, ConstructorApp& app)
 {
     Gtk::RadioMenuItem& itm = NewManaged<Gtk::RadioMenuItem>(grp);
-    itm.signal_toggled().connect(boost::lambda::bind(OnGoItemToggled, boost::ref(itm), boost::ref(app)));
+    itm.signal_toggled().connect( bb::bind(OnGoItemToggled, boost::ref(itm), boost::ref(app)) );
 
     return itm;
 }
 
-static void OnContentPageAdd(ConstructorApp& app, guint page_num)
+static void OnContentPageAdd(Gtk::Widget* , guint page_num)
 {
+    ConstructorApp& app = Application();
     Gtk::Notebook& nbook = app.BookTabs();
     int cnt = nbook.get_n_pages();
     ASSERT_RTL( app.BookContent().get_n_pages() == cnt + 1 );
@@ -548,8 +442,9 @@ static void OnContentPageAdd(ConstructorApp& app, guint page_num)
     }
 }
 
-static void OnMenuNotebookSwitch(ConstructorApp& app, guint page_num)
+static void OnMenuNotebookSwitch(GtkNotebookPage* , guint page_num)
 {
+    ConstructorApp& app = Application();
     app.BookContent().set_current_page(page_num);
     //app.BookContent().get_nth_page(page_num)->child_focus(Gtk::DIR_TAB_FORWARD);
 
@@ -570,184 +465,14 @@ static Gtk::Menu& DetachMenuFromUI(RefPtr<Gtk::UIManager> mngr, const char* path
     return menu;
 }
 
-static void SetAppTitle(ConstructorApp& app, bool clear_change_flag)
-{
-    if( clear_change_flag )
-        app.isProjectChanged = false;
-
-    const char* ch_flag = app.isProjectChanged ? "*" : "" ;
-    app.win.set_title(ch_flag + MakeProjectTitle(true) + " - " APROJECT_NAME);
-}
-
-static void ClearStore(RefPtr<ObjectStore> os)
-{
-    Gtk::TreeModel::Children children = os->children();
-    // с конца быстрее - не требуется реиндексация
-    while( children.size() )
-        DeleteMedia(os, --children.end());
-}
-
-static void NewProject()
-{
-    // * очищаем списки
-    AStores& as = GetAStores();
-    ClearStore(as.mnStore);
-    ClearStore(as.mdStore);
-    // * остальное в базе
-    AData().ClearSettings();
-}
-
-static void OnSaveAsProject(ConstructorApp& app);
-static void OnSaveProject(ConstructorApp& app);
-
-// предложить пользователю сохранить измененный проект перед закрытием
-static bool CheckBeforeClosing(ConstructorApp& app)
-{
-    bool res = true;
-    if( app.isProjectChanged )
-    {
-    
-        Gtk::MessageDialog dlg("<span weight=\"bold\" size=\"large\">" +
-                               BF_("Save changes to \"%1%\"?") % MakeProjectTitle() % bf::stop + 
-                               "</span>", true,  Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
-        dlg.add_button(_("Close _without Saving"), Gtk::RESPONSE_CLOSE);
-        AddCancelSaveButtons(dlg);
-    
-        bool to_save = false;
-        Gtk::ResponseType resp = (Gtk::ResponseType)dlg.run();
-        switch( resp )
-        {
-        case Gtk::RESPONSE_CANCEL:
-        case Gtk::RESPONSE_DELETE_EVENT:
-            res = false;
-            break;
-        case Gtk::RESPONSE_OK:
-        case Gtk::RESPONSE_CLOSE:
-            to_save = resp == Gtk::RESPONSE_OK;
-            break;
-        default:
-            ASSERT(0);
-        }
-
-        if( to_save )
-            OnSaveProject(app);
-    }
-    return res;
-}
-
-static Gtk::RadioButton& TVSelectionButton(bool &is_pal, bool val, Gtk::RadioButtonGroup& grp)
-{
-    const char* name = val ? "_PAL/SECAM" : "_NTSC" ;
-    Gtk::RadioButton& btn = *Gtk::manage(new Gtk::RadioButton(grp, name, true));
-    if( is_pal == val )
-        btn.set_active();
-    SetForRadioToggle(btn, bl::var(is_pal) = val);
-    return btn;
-}
-
-static void OnNewProject(ConstructorApp& app)
-{
-    if( CheckBeforeClosing(app) )
-    {
-        bool is_pal = Prefs().isPAL;
-        Gtk::Dialog new_prj_dlg(_("New Project"), app.win, true, true);
-        new_prj_dlg.set_name("NewProject");
-        new_prj_dlg.set_resizable(false);
-        {
-            AddCancelDoButtons(new_prj_dlg, Gtk::Stock::OK);
-            Gtk::VBox& dlg_box = *new_prj_dlg.get_vbox();
-            PackStart(dlg_box, NewManaged<Gtk::Image>((fs::path(GetDataDir())/"cap400.png").string()));
-            Gtk::VBox& vbox = Add(PackStart(dlg_box, NewPaddingAlg(10, 40, 20, 20)), NewManaged<Gtk::VBox>());
-            
-            PackStart(vbox, NewManaged<Gtk::Label>(_("Please select a Television standard for your project:"),
-                                                   0.0, 0.5, true));
-            {
-                Gtk::VBox& vbox2 = Add(PackStart(vbox, NewPaddingAlg(10, 10, 0, 0)), NewManaged<Gtk::VBox>());
-
-                Gtk::RadioButtonGroup grp;
-                PackStart(vbox2, TVSelectionButton(is_pal, true,  grp));
-                PackStart(vbox2, TVSelectionButton(is_pal, false, grp));
-            }
-            dlg_box.show_all();
-        }
-
-        if( Gtk::RESPONSE_OK == new_prj_dlg.run() )
-        {
-            NewProject();
-            AData().SetPalTvSystem(is_pal);
-            SetAppTitle(app);
-        }
-    }
-}
-
-static void OnOpenProject(ConstructorApp& app)
-{
-    if( CheckBeforeClosing(app) )
-    {
-        Gtk::FileChooserDialog dialog(_("Open Project"), Gtk::FILE_CHOOSER_ACTION_OPEN);
-        BuildChooserDialog(dialog, true, app.win);
-    
-        Gtk::FileFilter prj_filter;
-        prj_filter.set_name(_("Project files (*.xml)"));
-        prj_filter.add_pattern("*.xml");
-        dialog.add_filter(prj_filter);
-    
-        Gtk::FileFilter all_filter;
-        all_filter.set_name(_("All Files (*.*)"));
-        all_filter.add_pattern("*");
-        dialog.add_filter(all_filter);
-    
-        if( Gtk::RESPONSE_OK == dialog.run() )
-        {
-            // в процессе загрузки не нужен
-            dialog.hide();
-            IteratePendingEvents();
-    
-            NewProject();
-            AData().SetOut(false);
-
-            LoadApp(app, dialog.get_filename());
-        }
-    }
-}
-
-static void SaveProject(ConstructorApp& app)
-{
-    ASSERT( AData().IsProjectSet() );
-
-    AStores& as = GetAStores();
-    RefPtr<MenuStore> mn_store = as.mnStore;
-    SaveProjectData(mn_store, as.mdStore);
-    // очистка после сохранения
-    AData().SetOut(true);
-    for( MenuStore::iterator itr = mn_store->children().begin(), end = mn_store->children().end();
-         itr != end; ++itr )
-        ClearMenuSavedData(GetMenu(mn_store, itr));
-
-    SetAppTitle(app);
-}
-
-static void OnSaveProject(ConstructorApp& app)
-{
-    if( AData().IsProjectSet() )
-        SaveProject(app);
-    else
-        OnSaveAsProject(app);
-}
-
-static void OnSaveAsProject(ConstructorApp& app)
-{
-    if( SaveProjectAs(app.win) )
-        SaveProject(app);
-}
-
 static bool OnConstructorAppClose(ConstructorApp& app)
 {
     return !app.askSaveOnExit || CheckBeforeClosing(app);
 }
 
-static void QuitApplication(ConstructorApp& app)
+static void QuitApplication()
 {
+    ConstructorApp& app = Application();
     // согласно политике gtkmm окна Gtk::Window не уничтожаются; признаком
     // выхода из цикла является скрытие того окна, которое было аргументом
     // функции Gtk::Main::run() = "обработка очереди сообщений пока не скрыто окно"
@@ -755,24 +480,25 @@ static void QuitApplication(ConstructorApp& app)
         app.win.hide();
 }
 
-static bool OnDeleteApp(ConstructorApp& app)
+static bool OnDeleteApp(GdkEventAny* )
 {
-    QuitApplication(app);
+    QuitApplication();
     return true; // окно закрывает только QuitApplication()
 }
 
-static void OnChangeProject(ConstructorApp& app)
+static void OnChangeProject(MediaItem /*mi*/, const char* /*action*/)
 {
+    ConstructorApp& app = Application();
     if( !app.isProjectChanged )
     {
         app.isProjectChanged = true;
-        SetAppTitle(app, false);
+        SetAppTitle(false);
     }
 }
 
-static void ImportFromDVD(ConstructorApp& app)
+static void ImportFromDVD()
 {
-    DVD::RunImport(app.win);
+    DVD::RunImport(Application().win);
 }
 
 //////////////////////////////////////////////////
@@ -841,7 +567,7 @@ brasero_utils_get_size_string(gint64 dsize,
 		goto end;
 	}
 
-	remain = (size % 1024) / 100;
+	remain = (size % 1024) / 102.4;
 	size = size / 1024;
 	unit = BRASERO_UTILS_GO;
 
@@ -874,41 +600,46 @@ brasero_utils_get_size_string(gint64 dsize,
 					size);
 }
 
-static std::string ToSizeString(gint64 size)
+std::string ToSizeString(io::pos size, bool round)
 {
-    gchar* c_str = brasero_utils_get_size_string(size, TRUE, TRUE);
+    gchar* c_str = brasero_utils_get_size_string(size, TRUE, round ? TRUE : FALSE);
     std::string str_res = c_str;
     g_free(c_str);
 
     return str_res;
 }
 
-static void UpdateDVDSize(SizeBar& sz_bar)
+io::pos DVDPayloadSize()
 {
-    io::pos sz     = Author::VideoSizeSum();
+    SizeBar& sz_bar = Application().SB();
     io::pos typ_sz = DVDTypeSize((DVDType)sz_bar.dvdTypes.get_active_row_number());
 
+    // зарезервируем для IFO, BUP, VIDEO_TS-файлов
+    const int AUX_DVD_STUFF = 512*1024;
+    return typ_sz - AUX_DVD_STUFF;
+}
+
+void UpdateDVDSize()
+{
+    io::pos sz = ProjectSizeSum();
     std::string sz_str;
     if( sz != 0 )
     {
-        const int AUX_DVD_STUFF = 512*1024;
-        sz += AUX_DVD_STUFF; // зарезервируем для IFO, BUP, VIDEO_TS-файлов
+        sz_str = ToSizeString(sz, true);
+        io::pos typ_sz = DVDPayloadSize();
 
-        sz_str = ToSizeString(sz);
         if( sz > typ_sz )
             sz_str = "<span color=\"red\">" + sz_str + "</span>";
         sz_str += " of ";
     }
-    sz_bar.szLbl.set_markup(sz_str);
+    Application().SB().szLbl.set_markup(sz_str);
 }
 
 class UpdateDVDSizeVis: public ObjVisitor
 {
     public:
-    SizeBar& szBar;
-                       UpdateDVDSizeVis(SizeBar& sz_bar): szBar(sz_bar) {}
 
-    virtual      void  Visit(VideoMD& ) { UpdateDVDSize(szBar); }
+    virtual      void  Visit(VideoMD& ) { UpdateDVDSize(); }
 };
 
 void MuxAddStreams(const std::string& src_fname)
@@ -922,8 +653,6 @@ void MuxAddStreams(const std::string& src_fname)
 
 ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
 {
-    using namespace boost;
-    reference_wrapper<ConstructorApp> app_ref(*this);
     Add(win, vBox);
     // * область главного меню
     {
@@ -936,23 +665,17 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
         {
             RefPtr<Gtk::ActionGroup> prj_actions = Gtk::ActionGroup::create("ProjectActions");
 
-            prj_actions->add( Gtk::Action::create("ProjectMenu", "_Project") ); 
-            prj_actions->add( Gtk::Action::create("New",    Gtk::Stock::NEW,  _("_New Project")),
-                              Gtk::AccelKey("<control>N"), lambda::bind(&OnNewProject, app_ref) );
-            prj_actions->add( Gtk::Action::create("Open",   Gtk::Stock::OPEN, _("_Open...")),
-                              Gtk::AccelKey("<control>O"), lambda::bind(&OnOpenProject, app_ref) );
-            prj_actions->add( Gtk::Action::create("Save",   Gtk::Stock::SAVE, _("_Save")),
-                              Gtk::AccelKey("<control>S"), lambda::bind(&OnSaveProject, app_ref) );
-            prj_actions->add( Gtk::Action::create("SaveAs", Gtk::Stock::SAVE_AS, _("Save _As...")),
-                              lambda::bind(&OnSaveAsProject, app_ref) );
+            prj_actions->add( Gtk::Action::create("ProjectMenu", "_Project") );
+            AddSrlActions(prj_actions);
+
             prj_actions->add( Gtk::Action::create("Quit",   Gtk::Stock::QUIT, _("_Quit")), 
-                              Gtk::AccelKey("<control>Q"), lambda::bind(&QuitApplication, app_ref) );
+                              Gtk::AccelKey("<control>Q"), &QuitApplication );
             prj_actions->add( Gtk::Action::create("Import DVD", DOTS_("Add Videos from _DVD"), _("DVD-Import Assistant")), 
-                              lambda::bind(&ImportFromDVD, app_ref) );
+                              &ImportFromDVD );
             prj_actions->add( Gtk::Action::create("Mux Streams", DOTS_("_Mux"), _("Mux Elementary Streams into MPEG2")),
-                              bl::bind(&MuxAddStreams, std::string()) );
+                              bb::bind(&MuxAddStreams, std::string()) );
             prj_actions->add( Gtk::Action::create("Preferences", Gtk::Stock::PREFERENCES, DOTS_("Pr_eferences")), 
-                              bl::bind(&ShowPrefs, &win) );
+                              bb::bind(&ShowPrefs, &win) );
 
 
             RefPtr<Gtk::UIManager> mngr = Gtk::UIManager::create();
@@ -996,9 +719,16 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
         goMenu.append(CreateGoMenuItem(grp, *this)); // первый вставляем сразу
         
         // Help
-        Gtk::Menu& hlp_menu     = MakeSubmenu(AppendMI(main_bar, NewManaged<Gtk::MenuItem>(_("_Help"), true)));
-        Gtk::MenuItem& about_mi = AppendMI(hlp_menu, *Gtk::manage(new Gtk::ImageMenuItem(Gtk::Stock::ABOUT)));
-        about_mi.signal_activate().connect(lambda::bind(&OnDlgAbout, boost::ref(win)));
+        {
+            Gtk::Menu& hlp_menu = MakeSubmenu(AppendMI(main_bar, NewManaged<Gtk::MenuItem>(_("_Help"), true)));
+
+            Gtk::Image& hlp_img   = NewManaged<Gtk::Image>(Gtk::Stock::HELP, Gtk::ICON_SIZE_MENU);
+            Gtk::MenuItem& hlp_mi = AppendMI(hlp_menu, 
+                                             *Gtk::manage(new Gtk::ImageMenuItem(hlp_img, _("_Online Help"), true)));
+            hlp_mi.signal_activate().connect(&OnlineHelp);
+            Gtk::MenuItem& about_mi = AppendMI(hlp_menu, *Gtk::manage(new Gtk::ImageMenuItem(Gtk::Stock::ABOUT)));
+            about_mi.signal_activate().connect(&OnDlgAbout);
+        }
 
         // разведем меню и закладки на небольшое расстояние
         main_bar.show_all();
@@ -1021,13 +751,11 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
         bookTabs.property_tab_hborder() = 5;
         bookTabs.property_tab_vborder() = 1;
 
-        bookTabs.signal_expose_event().connect(
-            wrap_return<bool>(lambda::bind(&EraseTabLineOnExpose, boost::ref(bookTabs), lambda::_1, 
-					   true, (Gtk::Notebook*)0)) );
+        sig::connect(bookTabs.signal_expose_event(), 
+                     bb::bind(&EraseTabLineOnExpose, boost::ref(bookTabs), _1, true, (Gtk::Notebook*)0));
         //bookCont.set_show_border(false);
-        bookCont.signal_expose_event().connect(
-            wrap_return<bool>(lambda::bind(&EraseTabLineOnExpose, boost::ref(bookCont), 
-					   lambda::_1, false, &bookTabs)) );
+        sig::connect(bookCont.signal_expose_event(),
+                     bb::bind(&EraseTabLineOnExpose, boost::ref(bookCont), _1, false, &bookTabs));
 
         // * размер исходников
         Gtk::Label& lbl = PackStart(hbox, szBar.szLbl);
@@ -1047,13 +775,13 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
         dvd_types.set_focus_on_click(false);
         for( int i = 0; i<dvdCNT; i++ )
         {
-            std::string sz_str = ToSizeString(DVDTypeSize(DVDType(i)));
+            std::string sz_str = ToSizeString(DVDTypeSize(DVDType(i)), true);
             dvd_types.append_text("DVD " + sz_str);
         }
         dvd_types.set_active(dvdFIVE);
         // обработчики
-        dvd_types.signal_changed().connect( bl::bind(&UpdateDVDSize, boost::ref(szBar)));
-        MHandler mh = new UpdateDVDSizeVis(szBar);
+        dvd_types.signal_changed().connect(&UpdateDVDSize);
+        MHandler mh = new UpdateDVDSizeVis();
         RegisterOnInsert(mh);
         RegisterOnDelete(mh);
     }
@@ -1065,15 +793,40 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
     //bookCont.property_tab_pos()     = Gtk::POS_LEFT; // RIGHT;
     bookCont.property_show_tabs()   = false;
 
-    bookCont.signal_page_added().connect(lambda::bind(&OnContentPageAdd, app_ref, lambda::_2));
-    bookTabs.signal_switch_page().connect(lambda::bind(&OnMenuNotebookSwitch, app_ref, lambda::_2));
+    bookCont.signal_page_added().connect(&OnContentPageAdd);
+    bookTabs.signal_switch_page().connect(&OnMenuNotebookSwitch);
 
     //Gtk::PositionType pt = bookCont.get_tab_pos();
     //if( (pt == Gtk::POS_LEFT) || (pt == Gtk::POS_RIGHT) )
     //    bookCont.property_tab_hborder() = 1;
     
-    win.signal_delete_event().connect( wrap_return<bool>(lambda::bind(&OnDeleteApp, app_ref)) );
-    RegisterHook(lambda::bind(&OnChangeProject, app_ref));
+    win.signal_delete_event().connect(&OnDeleteApp);
+    RegisterHook(&OnChangeProject);
+}
+
+void SetTabName(Gtk::Notebook& nbook, const std::string& name, int pos)
+{
+    Gtk::Label& lbl = NewBoldItalicLabel(name, true);
+
+    // попытка сделать вертикальные закладки читабельными
+    //switch( nbook.get_tab_pos() )
+    //{
+    //case Gtk::POS_LEFT:
+    //case Gtk::POS_RIGHT:
+    //    lbl.set_angle(+90.);
+    //    {
+    //        RefPtr<Pango::Layout> lay = lbl.get_layout();
+    //        //lay->set_spacing(0);
+    //        PangoContext* cnxt = lay->get_context()->gobj();
+    //        pango_context_set_base_gravity(cnxt, PANGO_GRAVITY_EAST);
+    //        pango_context_set_gravity_hint(cnxt, PANGO_GRAVITY_HINT_STRONG);
+    //    }
+    //    break;
+    //default:
+    //    ;
+    //}
+
+    nbook.set_tab_label(*nbook.get_nth_page(pos), lbl);
 }
 
 void ConstructorApp::SetTabName(const std::string& name, int pos)
@@ -1082,13 +835,43 @@ void ConstructorApp::SetTabName(const std::string& name, int pos)
     GetNthGo(*this, pos)->add(NewManaged<Gtk::Label>(name, Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, true));
 }
 
+static bool UpdateAppSizes(GdkEventConfigure* event)
+{
+    Gtk::Window& win = Application().win;
+    UnnamedPreferences& up = UnnamedPrefs();
+
+    up.appSz.x = event->width;
+    up.appSz.y = event->height;
+
+    win.get_position(up.appPos.x, up.appPos.y);
+    return false;
+}
+
 ActionFunctor BuildConstructor(ConstructorApp& app, const std::string& prj_file_name)
 {
     // *
     InitAStores();
     // *
-    LoadApp(app, prj_file_name);
-    SetAppWindowSize(app.win, 1000);
+    LoadApp(prj_file_name);
+
+    // * установка размеров и положения программы
+    // не смотря на то, что документация повторяет N>5 раз (см., например, 
+    // gtk_window_get_position()), что WM должен устанавливать положение (а лучше и размеры)
+    // окон на экране, в реальности все по-другому (всем по^Wнас^Wвсе равно); в то время, когда
+    // народ жаждет запоминания расположения программ на рабочем столе (http://brainstorm.ubuntu.com/idea/1442/),
+    // разработчики Gnome перекладывают ответственность на разработчиков приложений, последним
+    // угрожают,- "ничего не трогать, это забота WM" (Rhythmbox, Evince сделали вид, плохо слышат); 
+    // а WM-ы ничего не могут и не хотят (без помощи от разработчиков приложений),- эпический провал (c)
+    // 
+    // Отписался по проблеме тут: https://bugzilla.gnome.org/show_bug.cgi?id=79285
+    UnnamedPreferences& up = UnnamedPrefs();
+    // используем мягкую форму вместо gtk_window_resize(), чтобы 
+    // не уродовали наш продукт (два раза ку) уменьшением размеров до нуля
+    app.win.set_default_size(up.appSz.x, up.appSz.y);
+    if( up.isLoaded )
+        app.win.move(up.appPos.x, up.appPos.y);
+    app.win.signal_configure_event().connect(&UpdateAppSizes, false);
+    
     // *
     AStores& as = GetAStores();
     RefPtr<MediaStore> md_store = as.mdStore;
@@ -1115,20 +898,26 @@ void RunConstructor(const std::string& prj_file_name, bool ask_save_on_exit)
     LoadPrefs();
     AData().SetPalTvSystem(Prefs().isPAL);
 
-    // *
-    ConstructorApp app;
-    app.askSaveOnExit = ask_save_on_exit;
-
-    std::list<RefPtr<Gdk::Pixbuf> > pix_lst;
-    static const fs::directory_iterator end_itr;
-    for( fs::directory_iterator itr(fs::path(GetDataDir())/"icons");
-        itr != end_itr; ++itr )
-        pix_lst.push_back(Gdk::Pixbuf::create_from_file(itr->string()));
-    Gtk::Window::set_default_icon_list(pix_lst);
-
-    ActionFunctor after_fnr = BuildConstructor(app, prj_file_name);
-    RunWindow(app.win);
-    after_fnr();
+    {
+        SingletonStack<ConstructorApp> app_ssi;
+        SingletonStack<MEditorArea> edt_ssi;
+        // *
+        ConstructorApp& app = Application();
+        app.askSaveOnExit = ask_save_on_exit;
+    
+        std::list<RefPtr<Gdk::Pixbuf> > pix_lst;
+        static const fs::directory_iterator end_itr;
+        for( fs::directory_iterator itr(DataDirPath("icons"));
+            itr != end_itr; ++itr )
+            pix_lst.push_back(Gdk::Pixbuf::create_from_file(itr->string()));
+        Gtk::Window::set_default_icon_list(pix_lst);
+    
+        ActionFunctor after_fnr = BuildConstructor(app, prj_file_name);
+        RunWindow(app.win);
+        after_fnr();
+    }
+    // сохраняем настройки после закрытия (=> обновления информации) окон
+    SaveUnnamedPrefs();
 }
 
 } // namespace Project
@@ -1144,11 +933,10 @@ void InitI18n()
         const char* prefix = GetInstallPrefix();
         std::string locale_prefix(prefix ? (fs::path(prefix) / "share" / "locale").string() : "build/po/locale");
         bindtextdomain("bombono-dvd", locale_prefix.c_str());
+        // результат gettext() должен быть в UTF-8, а не в пользовательской локали
+        // (проблемы с не UTF8-локалями вроде ru_RU.KOI8-R)
+        bind_textdomain_codeset("bombono-dvd", "UTF-8");
         textdomain("bombono-dvd");
     }
 }
 
-void GoUrl(const gchar* url)
-{
-    Project::go_url_show(0, url, 0);
-}
