@@ -2,7 +2,7 @@
 // mgui/tests/test_main.cpp
 // This file is part of Bombono DVD project.
 //
-// Copyright (c) 2007-2008 Ilya Murav'jov
+// Copyright (c) 2007-2010 Ilya Murav'jov
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 // Использовать PCH в этом файле бесполезно, так как
 // вначале нужно определить #define BOOST_AUTO_TEST_MAIN
 
-#define BOOST_AUTO_TEST_MAIN
+#include <mlib/tests/turn_on_test_main.h>
 #include <boost/test/auto_unit_test.hpp>
 
