@@ -158,7 +158,7 @@ TrackLayout::TrackLayout(Timeline::Monitor& mon):
     // :TRICKY:
     //get_window()->set_debug_updates();
 
-    if( !GetMonitor().GetPlayer().IsOpened() )
+    if( !GetMonitor().GetViewer().IsOpened() )
         CloseTrackLayout(*this);
 }
 
@@ -196,12 +196,12 @@ static void SetAdjBounds(Gtk::Adjustment& adj, int size, int page_size)
 
 double TrackLayout::FrameFPS()
 {
-    return Mpeg::GetFrameFPS(trkMon.GetPlayer());
+    return ::FrameFPS(trkMon.GetViewer());
 }
 
 double TrackLayout::GetFramesLength() 
 { 
-    return Timeline::GetFramesLength(trkMon.GetPlayer()); 
+    return FramesLength(trkMon.GetViewer()); 
 }
 
 int TrackLayout::GetGroundHeight()
@@ -374,10 +374,9 @@ Gtk::Button* CreateScaleButton(TrackLayout& trk_lay, bool is_plus)
     Gtk::Button* btn = Gtk::manage(new SquareButton);
     SetNonActiveButton(*btn, "ScaleTrackButton");
 
-    using namespace boost;
     int add = is_plus ? 1 : -1 ;
     btn->signal_clicked().connect( 
-        wrap_return<void>(lambda::bind(&MoveAdj, boost::ref(trk_lay.TrkScale()), add, true)) );
+        wrap_return<void>(bb::bind(&MoveAdj, boost::ref(trk_lay.TrkScale()), add, true)) );
 
     return btn;
 }
@@ -406,12 +405,14 @@ void TLOnDeleteVis::Visit(Project::VideoChapterMD& obj)
         RedrawDVDMark(layout, Project::ChapterPosInt(&obj));
 }
 
+Project::VideoItem SetCurrentVideo(Project::VideoItem new_vd);
+
 void TLOnDeleteVis::Visit(Project::VideoMD& obj)
 {
     if( &obj == CurrVideo )
     {
         CloseTrackLayout(layout);
-        Timeline::SetCurrentVideo(0);
+        SetCurrentVideo(0);
     }
 }
 
@@ -524,7 +525,7 @@ void PackTrackLayout(Gtk::Container& contr, TrackLayout& layout)
         Gtk::Button* dvd_btn = Gtk::manage(new Gtk::Button);
         SetNonActiveButton(*dvd_btn, "DVDLabelButton");
         SetTip(*dvd_btn, _("Add Chapter Point"));
-        dvd_btn->signal_clicked().connect( boost::lambda::bind(&InsertDVDMark, boost::ref(layout)) );
+        dvd_btn->signal_clicked().connect( bb::bind(&InsertDVDMark, boost::ref(layout)) );
 
         Gtk::Image* dvd_img = Gtk::manage(new Gtk::Image(GetFactoryImage("dvdmark.png")));
         dvd_btn->add(*dvd_img);
@@ -592,17 +593,14 @@ void CloseTrackLayout(TrackLayout& layout)
 
     if( layout.is_sensitive() ) // так определяем что мы в нормальном состоянии
     {
-        Monitor& mon       = layout.GetMonitor();
-        Mpeg::Player& plyr = mon.GetPlayer();
-
-        plyr.Close();
+        layout.GetMonitor().GetViewer().Close();
         layout.SetPos(-1);
 
         MakeFullUpdate(layout, false);
     }
 }
 
-Project::VideoItem Timeline::SetCurrentVideo(Project::VideoItem new_vd)
+Project::VideoItem SetCurrentVideo(Project::VideoItem new_vd)
 {
     Project::VideoItem old_vd = CurrVideo;
     CurrVideo = new_vd ? new_vd : Project::VideoItem(new Project::VideoMD) ;
@@ -610,7 +608,7 @@ Project::VideoItem Timeline::SetCurrentVideo(Project::VideoItem new_vd)
     return old_vd;
 }
 
-bool OpenTrackLayout(TrackLayout& layout, Project::VideoItem vd, TLFunctor on_open_fnr)
+bool OpenTrackLayout(TrackLayout& layout, Project::VideoItem vd, std::string& err)
 {
     bool res = false;
     if( CurrVideo == vd )
@@ -619,10 +617,8 @@ bool OpenTrackLayout(TrackLayout& layout, Project::VideoItem vd, TLFunctor on_op
     {
         CloseTrackLayout(layout);
 
-        using namespace Timeline;
-        Monitor& mon       = layout.GetMonitor();
-        Mpeg::Player& plyr = mon.GetPlayer();
-        if( plyr.Open(GetFilename(*vd).c_str()) )
+        Monitor& mon = layout.GetMonitor();
+        if( mon.GetViewer().Open(GetFilename(*vd).c_str(), err) )
         {
             NormalTL& stt = NormalTL::Instance();
             stt.ChangeTo(layout);
@@ -635,9 +631,6 @@ bool OpenTrackLayout(TrackLayout& layout, Project::VideoItem vd, TLFunctor on_op
     }
 
     SetCurrentVideo(res ? vd : Project::VideoItem());
-    if( res && on_open_fnr )
-        on_open_fnr(layout);
     return res;
 }
 
-
