@@ -2,7 +2,7 @@
 // mgui/timeline/mviewer.h
 // This file is part of Bombono DVD project.
 //
-// Copyright (c) 2008-2009 Ilya Murav'jov
+// Copyright (c) 2008-2010 Ilya Murav'jov
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #ifndef __MGUI_TIMELINE_MVIEWER_H__
 #define __MGUI_TIMELINE_MVIEWER_H__
 
-#include "layout.h" 
+#include <mlib/function.h> 
 
 typedef boost::function<void(const char*, Gtk::FileChooser&)> OpenFileFnr;
 // add_open_button = true - для просмотрщика mviewer
